"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const inject_1 = require("@furystack/inject");
const logging_1 = require("@furystack/logging");
const utils_1 = require("@furystack/utils");
const core_1 = require("@furystack/core");
const models_1 = require("./models");
const http_api_1 = require("./http-api");
const http_user_context_1 = require("./http-user-context");
// tslint:disable:max-classes-per-file
describe('HttpApi tests', () => {
    const getInjector = () => {
        const i = new inject_1.Injector();
        i.setupStores(sm => sm
            .addStore(new core_1.InMemoryStore({ model: core_1.User, primaryKey: 'username' }))
            .addStore(new core_1.InMemoryStore({ model: models_1.DefaultSession, primaryKey: 'sessionId' })));
        return i;
    };
    it('Can be constructed', async () => {
        await utils_1.usingAsync(getInjector(), async (i) => {
            i.useHttpApi();
            expect(i.getInstance(http_api_1.HttpApi)).toBeInstanceOf(http_api_1.HttpApi);
        });
    });
    it('NotFound Action is executed when no other action is awailable', done => {
        utils_1.usingAsync(getInjector(), async (i) => {
            const ExampleNotFoundAction = async () => {
                done();
                return models_1.EmptyResult();
            };
            i.useLogging(logging_1.LoggerCollection);
            i.useHttpApi({
                notFoundAction: ExampleNotFoundAction,
            });
            await i.getInstance(http_api_1.HttpApi).mainRequestListener({}, { sendActionResult: () => undefined });
        });
    });
    it('Action can be executed', done => {
        utils_1.usingAsync(getInjector(), async (i) => {
            const ExampleAction = async (injector) => {
                try {
                    const userContext = injector.getInstance(http_user_context_1.HttpUserContext);
                    injector.cachedSingletons.has(userContext.constructor);
                    done();
                }
                catch (error) {
                    done(error);
                }
                return models_1.EmptyResult();
            };
            i.useHttpApi({
                actions: [() => ExampleAction],
            });
            await i
                .getInstance(http_api_1.HttpApi)
                .mainRequestListener({ headers: {} }, { sendActionResult: () => undefined });
        });
    });
    it('Should throw error if multiple actions are resolved for a request', done => {
        utils_1.usingAsync(getInjector(), async (i) => {
            const ExampleAction = async () => {
                done();
                return models_1.EmptyResult();
            };
            i.useHttpApi({
                actions: [() => ExampleAction, () => ExampleAction],
            });
            try {
                await i.getInstance(http_api_1.HttpApi).mainRequestListener({}, { sendActionResult: () => undefined });
                done('Should throw error');
            }
            catch (error) {
                done();
            }
        });
    });
    it('Error Action is executed on other action errors executed', done => {
        utils_1.usingAsync(getInjector(), async (i) => {
            const ExampleFailAction = async () => {
                throw Error(':(');
            };
            const ExampleErrorAction = async () => {
                done();
                return models_1.EmptyResult();
            };
            i.setExplicitInstance(new logging_1.LoggerCollection());
            i.useHttpApi({
                actions: [() => ExampleFailAction],
                errorAction: ExampleErrorAction,
            });
            await i.getInstance(http_api_1.HttpApi).mainRequestListener({}, { sendActionResult: () => undefined });
        });
    });
});
//# sourceMappingURL=http-api.spec.js.map