"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const http_1 = require("http");
const core_1 = require("@furystack/core");
const inject_1 = require("@furystack/inject");
const utils_1 = require("@furystack/utils");
const uuid_1 = require("uuid");
const http_authentication_settings_1 = require("./http-authentication-settings");
/**
 * Injectable UserContext for FuryStack HTTP Api
 */
let HttpUserContext = class HttpUserContext {
    constructor(incomingMessage, serverResponse, injector, authentication, storeManager) {
        this.incomingMessage = incomingMessage;
        this.serverResponse = serverResponse;
        this.authentication = authentication;
        this.users = authentication.getUserStore(storeManager);
        this.sessions = authentication.getSessionStore(storeManager);
        this.logger = injector.logger.withScope(`@furystack/http-api/${this.constructor.name}`);
    }
    /**
     * Returns if the current user is authenticated
     */
    async isAuthenticated() {
        try {
            const currentUser = await this.getCurrentUser();
            return currentUser !== null;
        }
        catch (error) {
            return false;
        }
    }
    /**
     * Returns if the current user can be authorized with ALL of the specified roles
     *
     * @param roles The list of roles to authorize
     */
    async isAuthorized(...roles) {
        const currentUser = await this.getCurrentUser();
        for (const role of roles) {
            if (!currentUser || !currentUser.roles.some(c => c === role)) {
                return false;
            }
        }
        return true;
    }
    /**
     * Checks if the system contains a user with the provided name and password, throws an error otherwise
     *
     * @param userName The username
     * @param password The password
     */
    async authenticateUser(userName, password) {
        const match = (password &&
            password.length &&
            (await this.users.search({
                filter: {
                    username: { $eq: userName },
                    password: { $eq: this.authentication.hashMethod(password) },
                },
            }))) ||
            [];
        if (match.length === 1) {
            // eslint-disable-next-line no-shadow
            const { password, ...user } = match[0];
            return user;
        }
        await utils_1.sleepAsync(Math.random() * 1000);
        throw Error('Failed to authenticate.');
    }
    async getCurrentUser() {
        if (!this.user) {
            this.user = await this.authenticateRequest();
            return this.user;
        }
        return this.user;
    }
    getSessionIdFromRequest() {
        if (this.incomingMessage.headers.cookie) {
            const cookies = this.incomingMessage.headers.cookie
                .toString()
                .split(';')
                .filter(val => val.length > 0)
                .map(val => {
                const [name, value] = val.split('=');
                return { name: name.trim(), value: value.trim() };
            });
            const sessionCookie = cookies.find(c => c.name === this.authentication.cookieName);
            if (sessionCookie) {
                return sessionCookie.value;
            }
        }
        return null;
    }
    async authenticateRequest() {
        // Basic auth
        if (this.authentication.enableBasicAuth && this.incomingMessage.headers.authorization) {
            const authData = Buffer.from(this.incomingMessage.headers.authorization.toString().split(' ')[1], 'base64');
            const [userName, password] = authData.toString().split(':');
            return await this.authenticateUser(userName, password);
        }
        // Cookie auth
        const sessionId = this.getSessionIdFromRequest();
        if (sessionId) {
            const session = await this.sessions.get(sessionId);
            if (session) {
                const userResult = await this.users.search({
                    filter: {
                        username: { $eq: session.username },
                    },
                });
                if (userResult.length === 1) {
                    const { password, ...user } = userResult[0];
                    return user;
                }
                throw Error('Inconsistent session result');
            }
        }
        throw Error('Failed to authenticate request');
    }
    /**
     * Creates and sets up a cookie-based session for the provided user
     *
     * @param user The user to create a session for
     * @param serverResponse A serverResponse to set the cookie
     */
    async cookieLogin(user, serverResponse) {
        const sessionId = uuid_1.v1();
        await this.sessions.add({ sessionId, username: user.username });
        serverResponse.setHeader('Set-Cookie', `${this.authentication.cookieName}=${sessionId}; Path=/; HttpOnly`);
        this.logger.information({
            message: `User '${user.username}' logged in.`,
            data: {
                user,
                sessionId,
            },
        });
        return user;
    }
    async cookieLogout() {
        const sessionId = this.getSessionIdFromRequest();
        if (sessionId) {
            const user = await this.authenticateRequest();
            await this.sessions.remove(sessionId);
            this.serverResponse.setHeader('Set-Cookie', `${this.authentication.cookieName}=; Path=/; HttpOnly`);
            this.logger.information({
                message: `User '${user.username}' has been logged out.`,
                data: {
                    user,
                    sessionId,
                },
            });
        }
    }
};
HttpUserContext = tslib_1.__decorate([
    inject_1.Injectable({ lifetime: 'scoped' }),
    tslib_1.__metadata("design:paramtypes", [http_1.IncomingMessage,
        http_1.ServerResponse,
        inject_1.Injector,
        http_authentication_settings_1.HttpAuthenticationSettings,
        core_1.StoreManager])
], HttpUserContext);
exports.HttpUserContext = HttpUserContext;
//# sourceMappingURL=http-user-context.js.map