"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable prettier/prettier */
const http_1 = require("http");
const utils_1 = require("@furystack/utils");
const inject_1 = require("@furystack/inject");
const core_1 = require("@furystack/core");
const default_session_1 = require("./models/default-session");
const http_user_context_1 = require("./http-user-context");
require("./injector-extensions");
require("@furystack/logging");
exports.prepareInjector = (i) => {
    i.setupStores(sm => sm
        .addStore(new core_1.InMemoryStore({ model: core_1.User, primaryKey: 'username' }))
        .addStore(new core_1.InMemoryStore({ model: default_session_1.DefaultSession, primaryKey: 'sessionId' })));
    i.setExplicitInstance({ headers: {} }, http_1.IncomingMessage);
    i.setExplicitInstance({}, http_1.ServerResponse);
    i.useHttpApi().useHttpAuthentication();
};
describe('HttpUserContext', () => {
    const testUser = { username: 'testUser', roles: ['grantedRole1', 'grantedRole2'] };
    it('Should be constructed with the extension method', () => {
        utils_1.using(new inject_1.Injector(), i => {
            exports.prepareInjector(i);
            const ctx = i.getInstance(http_user_context_1.HttpUserContext);
            expect(ctx).toBeInstanceOf(http_user_context_1.HttpUserContext);
        });
    });
    describe('isAuthenticated', () => {
        it('Should return true for authenticated users', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                ctx.getCurrentUser = jest.fn(async () => testUser);
                const value = await ctx.isAuthenticated();
                expect(value).toBe(true);
                expect(ctx.getCurrentUser).toBeCalled();
            });
        });
        it('Should return false for unauthenticated users', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                ctx.getCurrentUser = jest.fn(async () => {
                    throw Error(':(');
                });
                await expect(ctx.isAuthenticated()).resolves.toEqual(false);
                expect(ctx.getCurrentUser).toBeCalled();
            });
        });
    });
    describe('isAuthorized', () => {
        it('Should return true if all roles are authorized', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                ctx.getCurrentUser = jest.fn(async () => testUser);
                const value = await ctx.isAuthorized('grantedRole1', 'grantedRole2');
                expect(value).toBe(true);
                expect(ctx.getCurrentUser).toBeCalled();
            });
        });
        it('Should return false if not all roles are authorized', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                ctx.getCurrentUser = jest.fn(async () => testUser);
                const value = await ctx.isAuthorized('grantedRole1', 'nonGrantedRole2');
                expect(value).toBe(false);
                expect(ctx.getCurrentUser).toBeCalled();
            });
        });
    });
    describe('authenticateUser', () => {
        it('Should fail when the store is empty', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                await expect(ctx.authenticateUser('user', 'password')).rejects.toThrow('');
            });
        });
        it('Should fail when the password not equals', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                ctx.authentication
                    .getUserStore(i.getInstance(core_1.StoreManager))
                    .add({ username: 'user', password: ctx.authentication.hashMethod('pass123'), roles: [] });
                await expect(ctx.authenticateUser('user', 'pass321')).rejects.toThrow('');
            });
        });
        it('Should fail when the username not equals', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                ctx.authentication
                    .getUserStore(i.getInstance(core_1.StoreManager))
                    .add({ username: 'otherUser', password: ctx.authentication.hashMethod('pass123'), roles: [] });
                expect(ctx.authenticateUser('user', 'pass123')).rejects.toThrow('');
            });
        });
        it('Should fail when password not provided', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                ctx.authentication
                    .getUserStore(i.getInstance(core_1.StoreManager))
                    .add({ username: 'otherUser', password: ctx.authentication.hashMethod('pass123'), roles: [] });
                await expect(ctx.authenticateUser('user', '')).rejects.toThrow('');
            });
        });
        it('Should return the user without the password hash when the username and password matches', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                const store = ctx.authentication.getUserStore(i.getInstance(core_1.StoreManager));
                const loginUser = { username: 'user', roles: [] };
                store.add({ ...loginUser, password: ctx.authentication.hashMethod('pass123') });
                const value = await ctx.authenticateUser('user', 'pass123');
                expect(value).toEqual(loginUser);
            });
        });
    });
    describe('getSessionIdFromRequest', () => {
        it('Should return null if no headers present', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                const sid = ctx.getSessionIdFromRequest();
                expect(sid).toBeNull();
            });
        });
        it('Should return null if no session ID cookie present', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                i.getInstance(http_1.IncomingMessage).headers = { cookie: 'a=2;b=3;c=4;' };
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                const sid = ctx.getSessionIdFromRequest();
                expect(sid).toBeNull();
            });
        });
        it('Should return the Session ID value if session ID cookie present', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                i.getInstance(http_1.IncomingMessage).headers = { cookie: `a=2;b=3;${ctx.authentication.cookieName}=666;c=4;` };
                const sid = ctx.getSessionIdFromRequest();
                expect(sid).toBe('666');
            });
        });
    });
    describe('authenticateRequest', () => {
        it('Should try to authenticate with Basic, if enabled', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                i.getInstance(http_1.IncomingMessage).headers = { authorization: `Basic dGVzdHVzZXI6cGFzc3dvcmQ=` };
                ctx.authenticateUser = jest.fn(async () => testUser);
                const result = await ctx.authenticateRequest();
                expect(ctx.authenticateUser).toBeCalledWith('testuser', 'password');
                expect(result).toBe(testUser);
            });
        });
        it('Should NOT try to authenticate with Basic, if disabled', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                ctx.authentication.enableBasicAuth = false;
                i.getInstance(http_1.IncomingMessage).headers = { authorization: `Basic dGVzdHVzZXI6cGFzc3dvcmQ=` };
                ctx.authenticateUser = jest.fn(async () => testUser);
                await expect(ctx.authenticateRequest()).rejects.toThrow('');
                expect(ctx.authenticateUser).not.toBeCalled();
            });
        });
        it('Should fail with no session in the store', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                i.getInstance(http_1.IncomingMessage).headers = { cookie: `${ctx.authentication.cookieName}=666;a=3` };
                await expect(ctx.authenticateRequest()).rejects.toThrow('');
            });
        });
        it('Should fail with valid session Id but no user', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                i.getInstance(http_1.IncomingMessage).headers = { cookie: `${ctx.authentication.cookieName}=666;a=3` };
                ctx.authentication
                    .getSessionStore(i.getInstance(core_1.StoreManager))
                    .add({ sessionId: '666', username: testUser.username });
                await expect(ctx.authenticateRequest()).rejects.toThrow('');
            });
        });
        it('Should authenticate with cookie, if the session IDs matches', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                i.getInstance(http_1.IncomingMessage).headers = { cookie: `${ctx.authentication.cookieName}=666;a=3` };
                ctx.authentication
                    .getSessionStore(i.getInstance(core_1.StoreManager))
                    .add({ sessionId: '666', username: testUser.username });
                ctx.authentication.getUserStore(i.getInstance(core_1.StoreManager)).add({ ...testUser, password: '' });
                const result = await ctx.authenticateRequest();
                expect(result).toEqual(testUser);
            });
        });
    });
    describe('getCurrentUser', () => {
        it('Should return the current user from authenticateRequest() once per request', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                ctx.authenticateRequest = jest.fn(async () => testUser);
                const result = await ctx.getCurrentUser();
                const result2 = await ctx.getCurrentUser();
                expect(ctx.authenticateRequest).toBeCalledTimes(1);
                expect(result).toBe(testUser);
                expect(result2).toBe(testUser);
            });
        });
    });
    describe('cookieLogin', () => {
        it('Should return the current user from authenticateRequest() once per request', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                const setHeader = jest.fn();
                ctx.sessions.add = jest.fn(async (s) => s);
                const authResult = await ctx.cookieLogin(testUser, { setHeader });
                expect(authResult).toBe(testUser);
                expect(setHeader).toBeCalled();
                expect(ctx.sessions.add).toBeCalled();
            });
        });
    });
    describe('cookieLogout', () => {
        it('Should invalidate the current session id cookie', async () => {
            await utils_1.usingAsync(new inject_1.Injector(), async (i) => {
                exports.prepareInjector(i);
                const ctx = i.getInstance(http_user_context_1.HttpUserContext);
                const setHeader = jest.fn();
                ctx.sessions.add = jest.fn(async (s) => s);
                ctx.authenticateRequest = jest.fn(async () => testUser);
                ctx.sessions.remove = jest.fn(async () => undefined);
                ctx.getSessionIdFromRequest = () => 'example-session-id';
                ctx.serverResponse.setHeader = jest.fn(() => undefined);
                await ctx.cookieLogin(testUser, { setHeader });
                await ctx.cookieLogout();
                expect(ctx.serverResponse.setHeader).toBeCalledWith('Set-Cookie', 'fss=; Path=/; HttpOnly');
                expect(ctx.sessions.remove).toBeCalled();
            });
        });
    });
});
//# sourceMappingURL=http-user-context.spec.js.map